"""Sorting templatetags."""
from django import template
from django.contrib.admin.utils import label_for_field
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from django_sorting_bootstrap.sort import sort_queryset

register = template.Library()


# based on contrib.admin.templatetags.admin_list.result_headers
def result_headers(context, cl):
    """Generates the list column headers."""
    for i, field_name in enumerate(cl.list_display):
        text, attr = label_for_field(field_name, cl.model, return_attr=True)
        if attr:
            # Potentially not sortable

            # if the field is the action checkbox: no sorting and special class
            if field_name == "action_checkbox":
                yield {
                    "text": text,
                    "class_attrib": mark_safe(' class="action-checkbox-column"'),
                    "sortable": False,
                }
                continue

        # OK, it is sortable if we got this far
        th_classes = ["sortable", "column-{}".format(field_name)]
        ascending = None
        is_sorted = False
        # Is it currently being sorted on?
        if context.get("current_sort_field") == str(i + 1):
            is_sorted = True
            ascending = False
            th_classes.append("sorted descending")
        elif context.get("current_sort_field") == "-" + str(i + 1):
            is_sorted = True
            ascending = True
            th_classes.append("sorted ascending")

        if "request" in context:
            url = context["request"].path
        else:
            url = "./"

        # TODO: when start using action_checkbox use i instead of i + 1. T
        # this +1 is to correct enumerate index
        url += "?sort_by="

        if ascending is False:
            url += "-"
        url += str(i + 1)

        if "getsortvars" in context:
            extra_vars = context["getsortvars"]
        else:
            if "request" in context:
                request = context["request"]
                getvars = request.GET.copy()
                if "sort_by" in getvars:
                    del getvars["sort_by"]
                if len(getvars.keys()) > 0:
                    context["getsortvars"] = "&%s" % getvars.urlencode()
                else:
                    context["getsortvars"] = ""
                extra_vars = context["getsortvars"]

        # append other vars to url
        url += extra_vars

        yield {
            "text": text,
            "url": url,
            "sortable": True,
            "sorted": is_sorted,
            "ascending": ascending,
            "class_attrib": format_html(' class="{}"', " ".join(th_classes))
            if th_classes
            else "",
        }


@register.inclusion_tag("sorting_bootstrap/sort_headers_frag.html", takes_context=True)
def sort_headers(context, cl):
    """Displays the headers and data list together."""
    headers = list(result_headers(context, cl))
    sorted_fields = False
    for h in headers:
        if h["sortable"] and h["sorted"]:
            sorted_fields = True
    return {"cl": cl, "result_headers": headers, "sorted_fields": sorted_fields}


# Usage: {% sort_link "text" "field_name" %}
# Usage: {% sort_link "text" "field_name" "Visible name" %}
def sort_link(context, text, sort_field, visible_name=None):
    """Sort links."""
    sorted_fields = False
    ascending = None
    class_attrib = "sortable"
    orig_sort_field = sort_field
    if context.get("current_sort_field") == sort_field:
        sort_field = "-%s" % sort_field
        visible_name = "-%s" % (visible_name or orig_sort_field)
        sorted_fields = True
        ascending = False
        class_attrib += " sorted descending"
    elif context.get("current_sort_field") == "-" + sort_field:
        visible_name = "%s" % (visible_name or orig_sort_field)
        sorted_fields = True
        ascending = True
        class_attrib += " sorted ascending"

    if visible_name:
        if "request" in context:
            request = context["request"]
            request.session[visible_name] = sort_field

    # builds url
    if "request" in context:
        url = context["request"].path
    else:
        url = "./"

    url += "?sort_by="
    if visible_name is None:
        url += sort_field
    else:
        url += visible_name

    if "getsortvars" in context:
        extra_vars = context["getsortvars"]
    else:
        if "request" in context:
            request = context["request"]
            getvars = request.GET.copy()
            if "sort_by" in getvars:
                del getvars["sort_by"]
            if len(getvars.keys()) > 0:
                context["getsortvars"] = "&%s" % getvars.urlencode()
            else:
                context["getsortvars"] = ""
            extra_vars = context["getsortvars"]

    # append other vars to url
    url += extra_vars

    return {
        "text": text,
        "url": url,
        "ascending": ascending,
        "sorted_fields": sorted_fields,
        "class_attrib": class_attrib,
    }


# registers the tags sort_link and sort_th with the same function sort_link
register.inclusion_tag(
    "sorting_bootstrap/sort_link_frag.html", takes_context=True, name="sort_link"
)(sort_link)
register.inclusion_tag(
    "sorting_bootstrap/sort_th_frag.html", takes_context=True, name="sort_th"
)(sort_link)


# Usage: {% auto_sort queryset %}
@register.tag
def auto_sort(parser, token):
    """Auto sort."""
    try:
        tag_name, queryset = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "{} tag requires a single argument".format(token.contents.split()[0])
        )
    return SortedQuerysetNode(queryset)


class SortedQuerysetNode(template.Node):
    """Sorted Queryset Node class."""

    def __init__(self, queryset):
        """Initialize."""
        self.queryset_var = queryset
        self.queryset = template.Variable(queryset)

    def render(self, context):
        queryset = self.queryset.resolve(context)
        if "request" in context:
            request = context["request"]
            queryset = sort_queryset(queryset, request, context)
        context[self.queryset_var] = queryset
        if "request" in context:
            getvars = request.GET.copy()
        else:
            getvars = {}
        if "sort_by" in getvars:
            context["current_sort_field"] = getvars["sort_by"]
            del getvars["sort_by"]
        if len(getvars.keys()) > 0:
            context["getsortvars"] = "&%s" % getvars.urlencode()
        else:
            context["getsortvars"] = ""
        return ""



"""Sort module."""


def sort_queryset(queryset, request, context=None):
    """Return a sorted queryset."""
    sort_by = request.GET.get("sort_by")
    if sort_by:
        if sort_by in [el.name for el in queryset.model._meta.fields]:
            queryset = queryset.order_by(sort_by)
        else:
            if sort_by in request.session:
                sort_by = request.session[sort_by]
                try:
                    queryset = queryset.order_by(sort_by)
                except Exception:
                    raise
            # added else to fix a bug when using changelist
            # TODO: use less ifs and more standard sorting
            elif context is not None:
                # sorted ascending
                if sort_by[0] != "-":
                    sort_by = context["cl"].list_display[int(sort_by) - 1]
                # sorted descending
                else:
                    sort_by = "-" + context["cl"].list_display[abs(int(sort_by)) - 1]
                queryset = queryset.order_by(sort_by)
    return queryset



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['django_sorting_bootstrap', 'django_sorting_bootstrap.templatetags']

package_data = \
{'': ['*'],
 'django_sorting_bootstrap': ['locale/de/LC_MESSAGES/*',
                              'templates/sorting_bootstrap/*']}

install_requires = \
['django>=2.0']

setup_kwargs = {
    'name': 'django-sorting-bootstrap',
    'version': '2.7.0',
    'description': 'Sorting templates API using sorting, Django templatetags and Bootstrap classes.',
    'long_description': "Django Sorting Bootstrap\n========================\n\n|PyPI| |Python Version| |License|\n\n|Read the Docs| |Tests|\n\n|pre-commit| |Black|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/django-sorting-bootstrap.svg\n   :target: https://pypi.org/project/django-sorting-bootstrap/\n   :alt: PyPI\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/django-sorting-bootstrap\n   :target: https://pypi.org/project/django-sorting-bootstrap\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/django-sorting-bootstrap\n   :target: https://opensource.org/licenses/MIT\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/django-sorting-bootstrap/latest.svg?label=Read%20the%20Docs\n   :target: https://django-sorting-bootstrap.readthedocs.io/\n   :alt: Read the documentation at https://django-sorting-bootstrap.readthedocs.io/\n.. |Tests| image:: https://github.com/staticdev/django-sorting-bootstrap/workflows/Tests/badge.svg\n   :target: https://github.com/staticdev/django-sorting-bootstrap/actions?workflow=Tests\n   :alt: Tests\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\n\nFeatures\n--------\n\nDjango Sorting Bootstrap is a pluggable mini-API to easy add sorting for querysets, links and table headers in Django_ templates. There is also a new tag that creates headers for sorting tables using Bootstrap_'s layout.\n\n\nRequirements\n------------\n\n* Python 3.7+\n* Django 2.0+\n* Bootstrap 3+\n\n\nInstallation\n------------\n\nYou can install *Django Sorting Bootstrap* via pip_ from PyPI_:\n\n.. code:: console\n\n   $ pip install django-sorting-bootstrap\n\n\nDocumentation\n-------------\n\nComplete instructions on installation and usage are found in ``docs`` directory and online at\nhttps://django-sorting-bootstrap.readthedocs.io.\n\n\nContributing\n------------\n\nContributions are very welcome.\nTo learn more, see the `Contributor Guide`_.\n\n\nLicense\n-------\n\nDistributed under the terms of the MIT_ license,\n*Django Sorting Bootstrap* is free and open source software.\n\n\nCredits\n-------\n\nThis app is based on `Agiliq's django-sorting`_ 0.1. It has two improvements over it: the new tags and the Twitter Bootstrap compliance idea.\n\n\n.. _issue here: https://github.com/staticdev/staticdev/issues\n.. _Django: https://www.djangoproject.com/\n.. _Bootstrap: http://getbootstrap.com/\n.. _MIT: http://opensource.org/licenses/MIT\n.. _PyPI: https://pypi.org/\n.. _pip: https://pip.pypa.io/\n.. _Agiliq's django-sorting: http://github.com/agiliq/django-sorting\n.. github-only\n.. _Contributor Guide: CONTRIBUTING.rst\n",
    'author': "Thiago D'Ávila",
    'author_email': 'staticdev-support@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/staticdev/django-sorting-bootstrap',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
